@@ -49,7 +49,6 @@ class MonthlyActiveUsersWorkerStore(RegistrationWorkerStore):
         hs: "HomeServer",
     ):
         super().__init__(database, db_conn, hs)
-        self._clock = hs.get_clock()
         self.hs = hs
 
         if hs.config.redis.redis_enabled:
@@ -226,7 +225,7 @@ class MonthlyActiveUsersWorkerStore(RegistrationWorkerStore):
                 reserved_users: reserved users to preserve
             """
 
-            thirty_days_ago = int(self._clock.time_msec()) - (1000 * 60 * 60 * 24 * 30)
+            thirty_days_ago = int(self.clock.time_msec()) - (1000 * 60 * 60 * 24 * 30)
 
             in_clause, in_clause_args = make_in_list_sql_clause(
                 self.database_engine, "user_id", reserved_users
@@ -304,9 +303,9 @@ class MonthlyActiveUsersWorkerStore(RegistrationWorkerStore):
             txn:
             threepids: List of threepid dicts to reserve
         """
-        assert (
-            self._update_on_this_worker
-        ), "This worker is not designated to update MAUs"
+        assert self._update_on_this_worker, (
+            "This worker is not designated to update MAUs"
+        )
 
         # XXX what is this function trying to achieve?  It upserts into
         # monthly_active_users for each *registered* reserved mau user, but why?
@@ -328,10 +327,10 @@ class MonthlyActiveUsersWorkerStore(RegistrationWorkerStore):
                         txn,
                         table="monthly_active_users",
                         keyvalues={"user_id": user_id},
-                        values={"timestamp": int(self._clock.time_msec())},
+                        values={"timestamp": int(self.clock.time_msec())},
                     )
             else:
-                logger.warning("mau limit reserved threepid %s not found in db" % tp)
+                logger.warning("mau limit reserved threepid %s not found in db", tp)
 
     async def upsert_monthly_active_user(self, user_id: str) -> None:
         """Updates or inserts the user into the monthly active user table, which
@@ -340,9 +339,9 @@ class MonthlyActiveUsersWorkerStore(RegistrationWorkerStore):
         Args:
             user_id: user to add/update
         """
-        assert (
-            self._update_on_this_worker
-        ), "This worker is not designated to update MAUs"
+        assert self._update_on_this_worker, (
+            "This worker is not designated to update MAUs"
+        )
 
         # Support user never to be included in MAU stats. Note I can't easily call this
         # from upsert_monthly_active_user_txn because then I need a _txn form of
@@ -379,9 +378,9 @@ class MonthlyActiveUsersWorkerStore(RegistrationWorkerStore):
             txn:
             user_id: user to add/update
         """
-        assert (
-            self._update_on_this_worker
-        ), "This worker is not designated to update MAUs"
+        assert self._update_on_this_worker, (
+            "This worker is not designated to update MAUs"
+        )
 
         # Am consciously deciding to lock the table on the basis that is ought
         # never be a big table and alternative approaches (batching multiple
@@ -391,7 +390,7 @@ class MonthlyActiveUsersWorkerStore(RegistrationWorkerStore):
             txn,
             table="monthly_active_users",
             keyvalues={"user_id": user_id},
-            values={"timestamp": int(self._clock.time_msec())},
+            values={"timestamp": int(self.clock.time_msec())},
         )
 
         self._invalidate_cache_and_stream(txn, self.get_monthly_active_count, ())
@@ -409,9 +408,9 @@ class MonthlyActiveUsersWorkerStore(RegistrationWorkerStore):
         Args:
             user_id: the user_id to query
         """
-        assert (
-            self._update_on_this_worker
-        ), "This worker is not designated to update MAUs"
+        assert self._update_on_this_worker, (
+            "This worker is not designated to update MAUs"
+        )
 
         if self._limit_usage_by_mau or self._mau_stats_only:
             # Trial users and guests should not be included as part of MAU group
