@@ -36,7 +36,6 @@ from typing import (
 )
 
 import attr
-from immutabledict import immutabledict
 
 from synapse.api.constants import EduTypes
 from synapse.replication.tcp.streams import ReceiptsStream
@@ -56,10 +55,10 @@ from synapse.types import (
     PersistedPosition,
     StrCollection,
 )
-from synapse.util import json_encoder
 from synapse.util.caches.descriptors import cached, cachedList
 from synapse.util.caches.stream_change_cache import StreamChangeCache
 from synapse.util.iterutils import batch_iter
+from synapse.util.json import json_encoder
 
 if TYPE_CHECKING:
     from synapse.server import HomeServer
@@ -125,6 +124,7 @@ class ReceiptsWorkerStore(SQLBaseStore):
         db_conn: LoggingDatabaseConnection,
         hs: "HomeServer",
     ):
+        super().__init__(database, db_conn, hs)
         self._instance_name = hs.get_instance_name()
 
         # In the worker store this is an ID tracker which we overwrite in the non-worker
@@ -139,6 +139,7 @@ class ReceiptsWorkerStore(SQLBaseStore):
             db_conn=db_conn,
             db=database,
             notifier=hs.get_replication_notifier(),
+            server_name=self.server_name,
             stream_name="receipts",
             instance_name=self._instance_name,
             tables=[("receipts_linearized", "instance_name", "stream_id")],
@@ -146,8 +147,6 @@ class ReceiptsWorkerStore(SQLBaseStore):
             writers=hs.config.worker.writers.receipts,
         )
 
-        super().__init__(database, db_conn, hs)
-
         max_receipts_stream_id = self.get_max_receipt_stream_id()
         receipts_stream_prefill, min_receipts_stream_id = self.db_pool.get_cache_dict(
             db_conn,
@@ -158,33 +157,16 @@ class ReceiptsWorkerStore(SQLBaseStore):
             limit=10000,
         )
         self._receipts_stream_cache = StreamChangeCache(
-            "ReceiptsRoomChangeCache",
-            min_receipts_stream_id,
+            name="ReceiptsRoomChangeCache",
+            server_name=self.server_name,
+            current_stream_pos=min_receipts_stream_id,
             prefilled_cache=receipts_stream_prefill,
         )
 
     def get_max_receipt_stream_id(self) -> MultiWriterStreamToken:
         """Get the current max stream ID for receipts stream"""
 
-        min_pos = self._receipts_id_gen.get_current_token()
-
-        positions = {}
-        if isinstance(self._receipts_id_gen, MultiWriterIdGenerator):
-            # The `min_pos` is the minimum position that we know all instances
-            # have finished persisting to, so we only care about instances whose
-            # positions are ahead of that. (Instance positions can be behind the
-            # min position as there are times we can work out that the minimum
-            # position is ahead of the naive minimum across all current
-            # positions. See MultiWriterIdGenerator for details)
-            positions = {
-                i: p
-                for i, p in self._receipts_id_gen.get_positions().items()
-                if p > min_pos
-            }
-
-        return MultiWriterStreamToken(
-            stream=min_pos, instance_map=immutabledict(positions)
-        )
+        return MultiWriterStreamToken.from_generator(self._receipts_id_gen)
 
     def get_receipt_stream_id_for_instance(self, instance_name: str) -> int:
         return self._receipts_id_gen.get_current_token_for_writer(instance_name)
@@ -762,7 +744,7 @@ class ReceiptsWorkerStore(SQLBaseStore):
 
             txn.execute(sql, args)
 
-            return [room_id for room_id, in txn]
+            return [room_id for (room_id,) in txn]
 
         results: List[str] = []
         for batch in batch_iter(room_ids, 1000):
@@ -1030,9 +1012,7 @@ class ReceiptsWorkerStore(SQLBaseStore):
             SELECT event_id WHERE room_id = ? AND stream_ordering IN (
                 SELECT max(stream_ordering) WHERE %s
             )
-        """ % (
-            clause,
-        )
+        """ % (clause,)
 
         txn.execute(sql, [room_id] + list(args))
         rows = txn.fetchall()
@@ -1093,7 +1073,7 @@ class ReceiptsWorkerStore(SQLBaseStore):
         if event_ts is None:
             return None
 
-        now = self._clock.time_msec()
+        now = self.clock.time_msec()
         logger.debug(
             "Receipt %s for event %s in %s (%i ms old)",
             receipt_type,
